"""
Main module for checking deprecated dependencies.
"""

from pathlib import Path
from typing import Dict, List, Any, Optional
from dataclasses import dataclass
from packaging import version

from .parser import DependencyParser
from .database import DeprecatedPackageDB


@dataclass
class DeprecatedPackage:
    """Information about deprecated package."""
    name: str
    current_version: str
    file_source: str
    deprecated_since: str
    reason: str
    alternatives: List[Dict[str, str]]
    needs_update: bool = False
    required_version: Optional[str] = None


@dataclass
class CheckResult:
    """Result of project check."""
    deprecated_packages: List[DeprecatedPackage]
    safe_packages: List[Dict[str, str]]
    total_deprecated: int
    total_safe: int
    files_checked: List[str]


class DeprecatedChecker:
    """Main class for checking deprecated dependencies."""
    
    def __init__(self, db_path: Optional[Path] = None):
        self.parser = DependencyParser()
        self.db = DeprecatedPackageDB(db_path)
    
    def check_project(self, project_path: Path) -> CheckResult:
        """Checks project for deprecated dependencies."""
        if not project_path.exists():
            raise FileNotFoundError(f"Path {project_path} does not exist")
        
        # Parse all dependency files
        dependencies_by_file = self.parser.parse_all_files(project_path)
        
        deprecated_packages = []
        safe_packages = []
        
        # Check each dependency
        for file_name, dependencies in dependencies_by_file.items():
            for package_name, package_version in dependencies:
                # Extract version from version specification string
                version_str = self._extract_version(package_version)
                
                # Check if package is deprecated
                dep_info = self.db.check_version_compatibility(package_name, version_str)
                
                if dep_info["is_deprecated"]:
                    deprecated_pkg = DeprecatedPackage(
                        name=package_name,
                        current_version=version_str or "not specified",
                        file_source=file_name,
                        deprecated_since=dep_info.get("deprecated_since", "unknown"),
                        reason=dep_info.get("reason", "not specified"),
                        alternatives=dep_info.get("alternatives", []),
                        needs_update=dep_info.get("needs_update", False),
                        required_version=dep_info.get("required_version")
                    )
                    deprecated_packages.append(deprecated_pkg)
                else:
                    safe_packages.append({
                        "name": package_name,
                        "version": version_str or "not specified",
                        "file_source": file_name
                    })
        
        return CheckResult(
            deprecated_packages=deprecated_packages,
            safe_packages=safe_packages,
            total_deprecated=len(deprecated_packages),
            total_safe=len(safe_packages),
            files_checked=list(dependencies_by_file.keys())
        )
    
    def _extract_version(self, version_spec: str) -> str:
        """Extracts version from version specification string."""
        if not version_spec:
            return ""
        
        # Remove comparison operators
        version_str = version_spec.lstrip("<>!=~")
        
        # If version contains range, take minimum
        if "," in version_str:
            parts = version_str.split(",")
            for part in parts:
                part = part.strip()
                if part.startswith(">=") or not part.startswith(("<", ">")):
                    return part.lstrip("<>!=~")
            return parts[0].strip().lstrip("<>!=~")
        
        return version_str
    
    def get_recommendations(self, result: CheckResult) -> List[Dict[str, Any]]:
        """Generates recommendations for updating."""
        recommendations = []
        
        for pkg in result.deprecated_packages:
            rec = {
                "package": pkg.name,
                "current_version": pkg.current_version,
                "file_source": pkg.file_source,
                "reason": pkg.reason,
                "alternatives": []
            }
            
            for alt in pkg.alternatives:
                alt_info = {
                    "name": alt["name"],
                    "reason": alt["reason"],
                    "migration_guide": alt.get("migration_guide", "")
                }
                
                if pkg.needs_update and alt["name"].lower() == pkg.name.lower():
                    alt_info["action"] = f"Update to version {pkg.required_version}+"
                else:
                    alt_info["action"] = "Replace with"
                
                rec["alternatives"].append(alt_info)
            
            recommendations.append(rec)
        
        return recommendations
    
    def generate_report(self, result: CheckResult, format_type: str = "text") -> str:
        """Generates report in specified format."""
        if format_type == "json":
            return self._generate_json_report(result)
        elif format_type == "yaml":
            return self._generate_yaml_report(result)
        else:
            return self._generate_text_report(result)
    
    def _generate_text_report(self, result: CheckResult) -> str:
        """Generates text report."""
        report = []
        report.append("Report on checking deprecated dependencies")
        report.append("=" * 50)
        report.append(f"Checked files: {', '.join(result.files_checked)}")
        report.append(f"Total packages: {result.total_deprecated + result.total_safe}")
        report.append(f"Deprecated: {result.total_deprecated}")
        report.append(f"Safe: {result.total_safe}")
        report.append("")
        
        if result.deprecated_packages:
            report.append("Found deprecated packages:")
            for pkg in result.deprecated_packages:
                report.append(f"  • {pkg.name}=={pkg.current_version} ({pkg.file_source})")
                report.append(f"    Reason: {pkg.reason}")
                if pkg.alternatives:
                    report.append("    Alternatives:")
                    for alt in pkg.alternatives:
                        report.append(f"      - {alt['name']}: {alt['reason']}")
                        if alt.get('migration_guide'):
                            report.append(f"        Guide: {alt['migration_guide']}")
                report.append("")
        else:
            report.append("No deprecated packages found!")
            report.append("")
        
        if result.safe_packages:
            report.append("Safe packages:")
            for pkg in result.safe_packages:
                report.append(f"  • {pkg['name']}=={pkg['version']} ({pkg['file_source']})")
        
        return "\n".join(report)
    
    def _generate_json_report(self, result: CheckResult) -> str:
        """Generates JSON report."""
        import json
        
        report_data = {
            "summary": {
                "total_packages": result.total_deprecated + result.total_safe,
                "deprecated_count": result.total_deprecated,
                "safe_count": result.total_safe,
                "files_checked": result.files_checked
            },
            "deprecated_packages": [
                {
                    "name": pkg.name,
                    "current_version": pkg.current_version,
                    "file_source": pkg.file_source,
                    "deprecated_since": pkg.deprecated_since,
                    "reason": pkg.reason,
                    "alternatives": pkg.alternatives,
                    "needs_update": pkg.needs_update,
                    "required_version": pkg.required_version
                }
                for pkg in result.deprecated_packages
            ],
            "safe_packages": result.safe_packages
        }
        
        return json.dumps(report_data, indent=2, ensure_ascii=False)
    
    def _generate_yaml_report(self, result: CheckResult) -> str:
        """Generates YAML report."""
        import yaml
        
        report_data = {
            "summary": {
                "total_packages": result.total_deprecated + result.total_safe,
                "deprecated_count": result.total_deprecated,
                "safe_count": result.total_safe,
                "files_checked": result.files_checked
            },
            "deprecated_packages": [
                {
                    "name": pkg.name,
                    "current_version": pkg.current_version,
                    "file_source": pkg.file_source,
                    "deprecated_since": pkg.deprecated_since,
                    "reason": pkg.reason,
                    "alternatives": pkg.alternatives,
                    "needs_update": pkg.needs_update,
                    "required_version": pkg.required_version
                }
                for pkg in result.deprecated_packages
            ],
            "safe_packages": result.safe_packages
        }
        
        return yaml.dump(report_data, default_flow_style=False, allow_unicode=True) 


"""
Scheduler for automatic database updates.
"""

import schedule
import time
import threading
from pathlib import Path
from typing import Optional, Callable
from datetime import datetime, timedelta
import logging
from dataclasses import dataclass

from .data_collector import DataCollector

logger = logging.getLogger(__name__)


@dataclass
class UpdateConfig:
    """Configuration for database updates."""
    enabled: bool = True
    interval_hours: int = 24  # Update every 24 hours
    retry_attempts: int = 3
    retry_delay_minutes: int = 30
    backup_before_update: bool = True
    notify_on_failure: bool = True


class DatabaseScheduler:
    """Scheduler for automatic database updates."""
    
    def __init__(self, config: Optional[UpdateConfig] = None):
        self.config = config or UpdateConfig()
        self.collector = DataCollector()
        self.is_running = False
        self.last_update = None
        self.update_thread = None
        
        # Path to file with information about last update
        self.status_file = Path(__file__).parent.parent / "cache" / "update_status.json"
        self.status_file.parent.mkdir(exist_ok=True)
    
    def start(self) -> None:
        """Starts the scheduler."""
        if self.is_running:
            logger.warning("Scheduler is already running")
            return
        
        self.is_running = True
        
        # Setup schedule
        schedule.every(self.config.interval_hours).hours.do(self._update_database)
        
        # Start in separate thread
        self.update_thread = threading.Thread(target=self._run_scheduler, daemon=True)
        self.update_thread.start()
        
        logger.info(f"Scheduler started with {self.config.interval_hours}h interval")
    
    def stop(self) -> None:
        """Stops the scheduler."""
        self.is_running = False
        schedule.clear()
        logger.info("Scheduler stopped")
    
    def _run_scheduler(self) -> None:
        """Runs the scheduler loop."""
        while self.is_running:
            schedule.run_pending()
            time.sleep(60)  # Check every minute
    
    def _update_database(self) -> None:
        """Performs database update with error handling."""
        logger.info("Starting scheduled database update...")
        
        # Create backup
        if self.config.backup_before_update:
            self._create_backup()
        
        # Try to update database
        for attempt in range(self.config.retry_attempts):
            try:
                self.collector.update_database()
                self._update_status(success=True)
                logger.info("Database update completed successfully")
                return
                
            except Exception as e:
                logger.error(f"Database update attempt {attempt + 1} failed: {e}")
                
                if attempt < self.config.retry_attempts - 1:
                    logger.info(f"Retrying in {self.config.retry_delay_minutes} minutes...")
                    time.sleep(self.config.retry_delay_minutes * 60)
                else:
                    self._update_status(success=False, error=str(e))
                    if self.config.notify_on_failure:
                        self._notify_failure(str(e))
    
    def _create_backup(self) -> None:
        """Creates a backup of the current database."""
        db_path = Path(__file__).parent.parent / "data" / "deprecated_packages.yaml"
        if db_path.exists():
            backup_path = Path(__file__).parent.parent / "cache" / f"backup_{datetime.now().strftime('%Y%m%d_%H%M%S')}.yaml"
            backup_path.parent.mkdir(exist_ok=True)
            
            import shutil
            shutil.copy2(db_path, backup_path)
            logger.info(f"Backup created: {backup_path}")
    
    def _update_status(self, success: bool, error: Optional[str] = None) -> None:
        """Updates the status file."""
        status = {
            "last_update": datetime.now().isoformat(),
            "success": success,
            "error": error,
            "next_update": (datetime.now() + timedelta(hours=self.config.interval_hours)).isoformat()
        }
        
        import json
        with open(self.status_file, 'w') as f:
            json.dump(status, f, indent=2)
        
        self.last_update = datetime.now()
    
    def _notify_failure(self, error: str) -> None:
        """Notifies about update failure."""
        # Here you can add notifications (email, Slack, etc.)
        logger.error(f"Database update failed: {error}")
    
    def force_update(self) -> bool:
        """Forces immediate database update."""
        logger.info("Forcing immediate database update...")
        
        try:
            self.collector.update_database()
            self._update_status(success=True)
            logger.info("Forced update completed successfully")
            return True
        except Exception as e:
            logger.error(f"Forced update failed: {e}")
            self._update_status(success=False, error=str(e))
            return False
    
    def get_status(self) -> dict:
        """Gets the current scheduler status."""
        status = {
            "is_running": self.is_running,
            "config": {
                "enabled": self.config.enabled,
                "interval_hours": self.config.interval_hours,
                "retry_attempts": self.config.retry_attempts
            },
            "last_update": self.last_update.isoformat() if self.last_update else None,
            "next_update": None
        }
        
        # Load information from status file
        if self.status_file.exists():
            import json
            try:
                with open(self.status_file, 'r') as f:
                    file_status = json.load(f)
                    status.update(file_status)
            except Exception as e:
                logger.warning(f"Error reading status file: {e}")
        
        return status
    
    def get_statistics(self) -> dict:
        """Gets statistics about the database."""
        return self.collector.get_statistics()


class ManualUpdater:
    """Manual updater for one-time database updates."""
    
    def __init__(self):
        self.collector = DataCollector()
    
    def update_from_source(self, source: str) -> bool:
        """Updates database from a specific source."""
        logger.info(f"Updating database from source: {source}")
        
        try:
            if source == "pypi":
                data = self.collector._collect_from_pypi()
            elif source == "manual":
                data = self.collector._collect_manual_data()
            elif source == "github":
                data = self.collector._collect_from_github()
            elif source == "security_advisories":
                data = self.collector._collect_security_advisories()
            else:
                logger.error(f"Unknown source: {source}")
                return False
            
            # Update database only with data from specified source
            db_path = Path(__file__).parent.parent / "data" / "deprecated_packages.yaml"
            existing_data = {}
            
            if db_path.exists():
                import yaml
                with open(db_path, 'r', encoding='utf-8') as f:
                    existing_data = yaml.safe_load(f) or {}
            
            # Update database only with data from specified source
            for package_name, package_data in data.items():
                package_data["source"] = source
                package_data["last_updated"] = datetime.now().isoformat()
                existing_data[package_name] = package_data
            
            # Save updated database
            with open(db_path, 'w', encoding='utf-8') as f:
                yaml.dump(existing_data, f, default_flow_style=False, allow_unicode=True)
            
            logger.info(f"Updated {len(data)} packages from {source}")
            return True
            
        except Exception as e:
            logger.error(f"Error updating from {source}: {e}")
            return False
    
    def validate_database(self) -> dict:
        """Validates the current database."""
        db_path = Path(__file__).parent.parent / "data" / "deprecated_packages.yaml"
        
        if not db_path.exists():
            return {"valid": False, "error": "Database file not found"}
        
        try:
            import yaml
            with open(db_path, 'r', encoding='utf-8') as f:
                data = yaml.safe_load(f)
            
            validation_result = {
                "valid": True,
                "total_packages": len(data),
                "sources": {},
                "errors": []
            }
            
            for package_name, package_data in data.items():
                # Check required fields
                required_fields = ["deprecated_since", "reason", "alternatives"]
                for field in required_fields:
                    if field not in package_data:
                        validation_result["errors"].append(
                            f"Package {package_name} missing required field: {field}"
                        )
                        validation_result["valid"] = False
                
                # Count sources
                source = package_data.get("source", "unknown")
                validation_result["sources"][source] = validation_result["sources"].get(source, 0) + 1
            
            return validation_result
            
        except Exception as e:
            return {"valid": False, "error": str(e)} 


"""
Database of deprecated packages and their alternatives.
"""

import yaml
from pathlib import Path
from typing import Dict, List, Optional, Any
from packaging import version
import importlib.resources as pkg_resources


class DeprecatedPackageDB:
    """Database of deprecated packages."""
    
    def __init__(self, db_path: Optional[Path] = None):
        if db_path is None:
            # Always try to load from package data first
            self.db_path = None  # Will load from package
        else:
            self.db_path = db_path
        
        self._load_database()
    
    def _load_database(self):
        """Loads database from YAML file."""
        try:
            if self.db_path is None:
                # Load from package data
                import pkg_resources
                try:
                    # Try different package names
                    package_names = ['deprecated-checker', 'deprecated_checker', 'core']
                    for package_name in package_names:
                        try:
                            with pkg_resources.resource_stream(package_name, 'data/deprecated_packages.yaml') as f:
                                self.data = yaml.safe_load(f) or {}
                                return
                        except Exception:
                            continue
                    
                    # Fallback to relative path
                    data_file = Path(__file__).parent.parent / "data" / "deprecated_packages.yaml"
                    with open(data_file, 'r', encoding='utf-8') as f:
                        self.data = yaml.safe_load(f) or {}
                except Exception as e:
                    print(f"Error loading database: {e}")
                    # Fallback to relative path
                    data_file = Path(__file__).parent.parent / "data" / "deprecated_packages.yaml"
                    with open(data_file, 'r', encoding='utf-8') as f:
                        self.data = yaml.safe_load(f) or {}
            else:
                # Load from file path
                with open(self.db_path, 'r', encoding='utf-8') as f:
                    self.data = yaml.safe_load(f) or {}
        except Exception as e:
            print(f"Error loading database: {e}")
            self.data = {}
    
    def is_deprecated(self, package_name: str, package_version: str = "") -> bool:
        """Checks if package is deprecated."""
        package_name = package_name.lower()
        return package_name in self.data
    
    def get_deprecated_info(self, package_name: str) -> Optional[Dict[str, Any]]:
        """Gets information about deprecated package."""
        package_name = package_name.lower()
        return self.data.get(package_name)
    
    def get_alternatives(self, package_name: str) -> List[Dict[str, str]]:
        """Gets list of alternatives for deprecated package."""
        info = self.get_deprecated_info(package_name)
        if info and 'alternatives' in info:
            return info['alternatives']
        return []
    
    def check_version_compatibility(self, package_name: str, current_version: str) -> Dict[str, Any]:
        """Checks version compatibility of package."""
        package_name = package_name.lower()
        info = self.get_deprecated_info(package_name)
        
        if not info:
            return {"is_deprecated": False}
        
        result = {
            "is_deprecated": True,
            "deprecated_since": info.get("deprecated_since"),
            "reason": info.get("reason"),
            "alternatives": info.get("alternatives", []),
            "current_version": current_version,
            "needs_update": False
        }
        
        # Check if package needs update
        if current_version and "alternatives" in info:
            for alt in info["alternatives"]:
                if alt["name"].lower() == package_name:
                    # This is update of the same package
                    if "version" in alt:
                        try:
                            current_ver = version.parse(current_version)
                            required_ver = version.parse(alt["version"])
                            if current_ver < required_ver:
                                result["needs_update"] = True
                                result["required_version"] = alt["version"]
                        except version.InvalidVersion:
                            pass
                    break
        
        return result
    
    def get_all_deprecated_packages(self) -> List[str]:
        """Returns list of all deprecated packages."""
        return list(self.data.keys())
    
    def search_alternatives(self, package_name: str) -> List[Dict[str, str]]:
        """Searches alternatives for package."""
        return self.get_alternatives(package_name)
    
    def get_migration_guide(self, package_name: str, alternative_name: str) -> Optional[str]:
        """Gets link to migration guide."""
        alternatives = self.get_alternatives(package_name)
        for alt in alternatives:
            if alt["name"].lower() == alternative_name.lower():
                return alt.get("migration_guide")
        return None
    
    def export_to_json(self) -> str:
        """Exports database to JSON format."""
        import json
        return json.dumps(self.data, indent=2, ensure_ascii=False)
    
    def export_to_yaml(self) -> str:
        """Exports database to YAML format."""
        return yaml.dump(self.data, default_flow_style=False, allow_unicode=True)
    
    def export_to_csv(self) -> str:
        """Exports database to CSV format."""
        import csv
        import io
        
        output = io.StringIO()
        writer = csv.writer(output)
        
        # Write header
        writer.writerow(["Package", "Deprecated Since", "Reason", "Alternatives"])
        
        # Write data
        for package_name, info in self.data.items():
            deprecated_since = info.get("deprecated_since", "")
            reason = info.get("reason", "")
            alternatives = ", ".join([alt["name"] for alt in info.get("alternatives", [])])
            
            writer.writerow([package_name, deprecated_since, reason, alternatives])
        
        return output.getvalue() 


#!/usr/bin/env python3
"""
Setup for Deprecated Checker.
"""

from setuptools import setup, find_packages
from pathlib import Path

# Read README
readme_path = Path(__file__).parent / "README.md"
long_description = ""
if readme_path.exists():
    with open(readme_path, "r", encoding="utf-8") as f:
        long_description = f.read()

# Read requirements
requirements_path = Path(__file__).parent / "requirements.txt"
install_requires = []
if requirements_path.exists():
    with open(requirements_path, "r", encoding="utf-8") as f:
        install_requires = [line.strip() for line in f if line.strip() and not line.startswith("#")]

setup(
    name="deprecated-checker",
    version="1.0.0",
    author="Iulian Pavlov",
    author_email="iulianpavlov@icloud.com",
    description="Tool for checking deprecated dependencies in Python projects",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/julicq/deprecated-checker",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Quality Assurance",
    ],
    python_requires=">=3.8",
    install_requires=install_requires,
    entry_points={
        "console_scripts": [
            "deprecated-checker=deprecated_checker:main",
        ],
    },
    include_package_data=True,
    package_data={
        "": ["data/*.yaml"],
    },
    keywords="python dependencies deprecated security audit",
    project_urls={
        "Bug Reports": "https://github.com/julicq/deprecated-checker/issues",
        "Source": "https://github.com/julicq/deprecated-checker",
        "Documentation": "https://github.com/julicq/deprecated-checker#readme",
    },
) 